@@ -45,6 +45,7 @@ from supervisor.datatypes import url
 from supervisor.datatypes import Automatic
 from supervisor.datatypes import auto_restart
 from supervisor.datatypes import profile_options
+from supervisor.datatypes import sig_pattern
 
 from supervisor import loggers
 from supervisor import states
@@ -846,6 +847,9 @@ class ServerOptions(Options):
         # system settings
         systemjob = boolean(get(section, 'systemjob', 'true'))
 
+        # The name of the shared socket file descriptor parameter (uvicorn --fd).
+        socket_fd_param = get(section, 'socket_fd_param', None)
+
         # default settings
         priority = integer(get(section, 'priority', 999))
         autostart = boolean(get(section, 'autostart', 'true'))
@@ -959,6 +963,7 @@ class ServerOptions(Options):
                 cpupriority=cpupriority,
                 cpuaffinity=cpuaffinity,
                 systemjob=systemjob,
+                socket_fd_param=socket_fd_param,
                 stdout_logfile=logfiles['stdout_logfile'],
                 stdout_capture_maxbytes=stdout_cmaxbytes,
                 stdout_events_enabled=stdout_events,
@@ -1240,7 +1245,8 @@ class ServerOptions(Options):
             pass
 
     def fork(self):
-        return 0  # os.fork()
+        """0 indicating spawn as child"""
+        return 0
 
     def dup2(self, frm, to):
         return os.dup2(frm, to)
@@ -1732,6 +1738,8 @@ class EventListenerConfig(ProcessConfig):
 
 
 class FastCGIProcessConfig(ProcessConfig):
+    optional_param_names = ['socket_fd_param'] + ProcessConfig.optional_param_names
+
     def make_process(self, group=None):
         if group is None:
             raise NotImplementedError('FastCGI programs require a group')
@@ -1949,12 +1957,9 @@ def signame(sig):
 def _init_signames():
     global _signames
     d = {}
-    for k, v in signal.__dict__.items():
-        k_startswith = getattr(k, "startswith", None)
-        if k_startswith is None:
-            continue
-        if k_startswith("SIG") and not k_startswith("SIG_"):
-            d[v] = k
+    for k in dir(signal):
+        if sig_pattern.match(k):
+            d[getattr(signal, k)] = k
     _signames = d
 
 
