@@ -330,8 +330,16 @@ class Subprocess(object):
             self.change_state(ProcessStates.BACKOFF)
             return
 
+        # for compatibility with the original api
+        forkpid = options.fork()
+
         self._spawn_as_child(filename, argv)
 
+        # under tests
+        if forkpid != 0 and not self.process:
+            self.process = DummyPopen()
+            self.pid = self.process.pid
+
         try:
             self.dispatchers, self.pipes = self.config.make_dispatchers(self)
         except (OSError, IOError) as why:
@@ -345,6 +353,10 @@ class Subprocess(object):
             self.record_spawnerr(msg)
             self._assertInState(ProcessStates.STARTING)
             self.change_state(ProcessStates.BACKOFF)
+            return
+
+        # returns the process pid. None if failed
+        return self.process.pid if self.process else None
 
     def _setup_system_resource(self):
         # Adds the process to the job
@@ -379,6 +391,8 @@ class Subprocess(object):
         """Runs a new process and returns its reference"""
         redirect_stderr = kwargs.pop('redirect_stderr', False)
         kwargs.update(dict(
+            # Prevents the supervisor from inheriting the signal when expected by the process.
+            creationflags=subprocess.CREATE_NEW_PROCESS_GROUP,
             universal_newlines=True,
             stdout=subprocess.PIPE,
             stdin=subprocess.PIPE
@@ -854,6 +868,27 @@ class FastCGISubprocess(Subprocess):
                                       '%s:%s' % (self.group, dir(self.group)))
         self.fcgi_sock = self.group.socket_manager.get_socket()
 
+    @property
+    def has_shared_socket(self):
+        """Whether socket sharing is possible"""
+        return self.fcgi_sock and self.config.socket_fd_param
+
+    def get_execv_args(self):
+        filename, commandargs = super(FastCGISubprocess, self).get_execv_args()
+        if self.has_shared_socket:
+            # passes the fd from the shared socket to the socket
+            socket_fd = self.fcgi_sock.fileno()
+            commandargs.extend([self.config.socket_fd_param, str(socket_fd)])
+        return filename, commandargs
+
+    def execute(self, filename, argv, **kwargs):
+        if self.has_shared_socket and not PY2:
+            # shares the fd with the fastcgi process
+            kwargs['startupinfo'] = subprocess.STARTUPINFO(lpAttributeList={
+                'handle_list': [self.fcgi_sock.fileno()]
+            })
+        return super(FastCGISubprocess, self).execute(filename, argv, **kwargs)
+
     def spawn(self):
         """
         Overrides Subprocess.spawn() so we can hook in before it happens
