@@ -45,29 +45,47 @@ try:
 except ImportError:
     sys.path.append(os.path.abspath(os.path.join(LOCAL_DIR, '..')))
 
-SUPERVISOR_PREFIX = "Supervisor Pyv{0.winver}".format(sys)
-
 
 class ConfigReg(object):
     """Saves the path to the supervisor.conf in the system registry"""
-    prefix = SUPERVISOR_PREFIX
+    service_name = "Supervisor Pyv{0.winver}".format(sys)
 
-    def __init__(self, prefix=None):
+    def __init__(self, service_name=None):
         self.software_key = winreg.OpenKey(winreg.HKEY_CURRENT_CONFIG, "Software")
-        if prefix is not None:
-            self.prefix = prefix
-        self.service_name_key = self.prefix + " Service"
+        if service_name is not None:
+            self.service_name = service_name
+        self.service_config_dir_key = self.service_name + " Service"
+        self.service_name_key = "Name"
         self.config_name_key = "Config"
 
-    def set(self, filepath):
-        with winreg.CreateKey(self.software_key, self.service_name_key) as srv_key:
-            winreg.SetValue(srv_key, self.config_name_key, winreg.REG_SZ, filepath)
-
-    def get(self):
-        with winreg.OpenKey(self.software_key, self.service_name_key) as srv_key:
-            value = winreg.QueryValue(srv_key, self.config_name_key)
+    def __getitem__(self, item):
+        """Get value from registry"""
+        with winreg.OpenKey(self.software_key, self.service_config_dir_key) as srv_key:
+            value = winreg.QueryValue(srv_key, item)
         return value
 
+    def __setitem__(self, key, value):
+        """Set value to registry"""
+        with winreg.CreateKey(self.software_key, self.service_config_dir_key) as srv_key:
+            winreg.SetValue(srv_key, key, winreg.REG_SZ, value)
+
+    @property
+    def filepath(self):
+        """get supervisor config path"""
+        return self[self.config_name_key]
+
+    @filepath.setter
+    def filepath(self, value):
+        """set supervisor config path"""
+        self[self.config_name_key] = value
+
+    def get(self, name, *args):
+        try:
+            return self[name]
+        except WindowsError:
+            if args:
+                return args[0]
+
     def close(self):
         """cleanup"""
         winreg.CloseKey(self.software_key)
@@ -79,9 +97,13 @@ class ConfigReg(object):
         self.close()
 
 
+config_reg = ConfigReg()
+
+
 class SupervisorService(win32serviceutil.ServiceFramework):
-    _svc_name_ = SUPERVISOR_PREFIX
-    _svc_display_name_ = "{0} process monitor".format(SUPERVISOR_PREFIX)
+    _svc_name_ = config_reg.get(config_reg.service_name_key,
+                                config_reg.service_name)
+    _svc_display_name_ = _svc_name_ + " process monitor"
     _svc_description_ = "A process control system"
     _svc_deps_ = []
 
@@ -95,29 +117,27 @@ class SupervisorService(win32serviceutil.ServiceFramework):
 
         # Gets the path of the registry configuration file
         try:
-            with ConfigReg() as reg:
-                self.config_filepath = reg.get()
+            config_reg_exc = None
+            self.supervisor_conf = config_reg.filepath
         except WindowsError:
             config_reg_exc = traceback.format_exc()
-            self.config_filepath = None
-        else:
-            config_reg_exc = None
+            self.supervisor_conf = None
 
         # The log goes to the location of the configuration file
-        if self.config_filepath is not None:
-            config_dir = os.path.dirname(self.config_filepath)
+        if self.supervisor_conf is not None:
+            config_dir = os.path.dirname(self.supervisor_conf)
         else:  # or to python home
             config_dir = os.getcwd()
 
         self.logger = logging.getLogger(__name__)
-        log_path = os.path.join(config_dir, SUPERVISOR_PREFIX.lower() + "-service.log")
+        log_path = os.path.join(config_dir, self._svc_name_.lower() + "-service.log")
         hdl = logging.handlers.RotatingFileHandler(log_path,
                                                    maxBytes=1024 ** 2,
                                                    backupCount=3)
         hdl.setFormatter(logging.Formatter("%(asctime)s [%(levelname)s] %(message)s"))
         self.logger.setLevel(logging.INFO)
         self.logger.addHandler(hdl)
-        self.logger.info("supervisor config path: {0!s}".format(self.config_filepath))
+        self.logger.info("supervisor config path: {0!s}".format(self.supervisor_conf))
 
         if config_reg_exc is not None:
             self.logger.error("* The service needs to be reinstalled")
@@ -125,11 +145,25 @@ class SupervisorService(win32serviceutil.ServiceFramework):
             logging.shutdown()
             exit(-1)
 
+    @classmethod
+    def set_service_name(cls, name):
+        cls._svc_name_ = name
+
+    @classmethod
+    def set_service_display_name(cls, name):
+        cls._svc_display_name_ = name
+
     @staticmethod
     def _get_fp_value(fp):
         """Return value of stream"""
         return fp.getvalue().strip("\n ")
 
+    @classmethod
+    def log(cls, fp, callback):
+        fp_value = cls._get_fp_value(fp)
+        if fp_value:
+            return callback(fp_value)
+
     # noinspection PyBroadException
     def SvcStop(self):
         self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
@@ -138,9 +172,9 @@ class SupervisorService(win32serviceutil.ServiceFramework):
         try:
             self.logger.info("supervisorctl shutdown")
             from supervisor import supervisorctl
-            supervisorctl.main(("-c", self.config_filepath, "shutdown"),
+            supervisorctl.main(("-c", self.supervisor_conf, "shutdown"),
                                stdout=stdout)
-            self.logger.info(self._get_fp_value(stdout))
+            self.log(stdout, self.logger.info)
         except SystemExit:
             pass  # normal exit
         except:
@@ -165,10 +199,10 @@ class SupervisorService(win32serviceutil.ServiceFramework):
         try:
             from supervisor import supervisord
             self.logger.info("supervisor starting...")
-            supervisord.main(("-c", self.config_filepath),
+            supervisord.main(("-c", self.supervisor_conf),
                              stdout=stdout, stderr=stderr)
-            self.logger.info(self._get_fp_value(stdout))
-            self.logger.error(self._get_fp_value(stderr))
+            self.log(stdout, self.logger.info)
+            self.log(stderr, self.logger.error)
         except:
             self.logger.exception("supervisor starting failed")
         finally:
@@ -181,27 +215,21 @@ def parse_args_config(options, argv):
     index = 0
     while True:
         try:
-            item = argv[index]
+            varg = argv[index]
+            last_index = index
+            index += 1
         except IndexError:
             break
         for opts in options:
-            if any(filter(item.startswith, opts['args'])):
+            if any([bool(re.match(re.escape(varg), n)) for n in opts['args']]):
+                index -= 1
                 name = argv.pop(index)
                 if name.find('=') > -1:
-                    args.extend(item.split('='))
-                    index -= 1
-                    continue
-                args.append(name)
-                kwargs = opts.setdefault('kwargs', {})
-                if kwargs.get('required', False):
-                    try:
-                        args.append(argv.pop(index))
-                        index -= 1
-                    except IndexError:
-                        continue
+                    args.extend(varg.split('='))
                 else:
-                    index -= 1
-        index += 1
+                    args.append(name)
+                    args.append(argv.pop(index))
+                index = last_index
     return args
 
 
@@ -209,12 +237,15 @@ def get_config_args(argv=None):
     argv = list(sys.argv) if argv is None else list(argv)
     options = [
         {'args': ('-h', '--help'),
-         'kwargs': {'required': False,
-                    'action': 'store_true'}},
+         'kwargs': {'required': False, 'action': 'store_true'}},
         {'args': ('-c', '--config'),
          'kwargs': {'type': argparse.FileType('r'),
                     'help': 'full filepath to supervisor.conf',
-                    'required': 'install' in argv}}
+                    'required': 'install' in argv}},
+        {'args': ('-sn', '--service-name'),
+         'kwargs': {'required': False, 'type': str}},
+        {'args': ('-sdn', '--service-display-name'),
+         'kwargs': {'required': False, 'type': str}},
     ]
     args = parse_args_config(options, argv)
     return options, args, argv
@@ -252,9 +283,17 @@ def main():
             parser.print_help(file=sys.stdout)
             print()
             srv_argv.append('-h')
+        # supervisor conf
         elif options.config:
-            with ConfigReg() as reg:
-                reg.set(options.config.name)
+            config_reg.filepath = options.config.name
+        # custom service name
+        if options.service_name:
+            config_reg[config_reg.service_name_key] = options.service_name
+            SupervisorService.set_service_name(options.service_name)  # runtime only
+            SupervisorService.set_service_display_name(options.service_name + " process monitor")
+        # custom service display name
+        if options.service_display_name:
+            SupervisorService.set_service_display_name(options.service_display_name)
         srv_argv.insert(0, sys.argv[0])
         win32serviceutil.HandleCommandLine(SupervisorService, argv=srv_argv)
 
@@ -264,6 +303,9 @@ if __name__ == '__main__':
         main()
     except (SystemExit, KeyboardInterrupt):
         raise
-    except:
+    except Exception as exc:
         print(" command execution failed ".center(35, '='))
+        if isinstance(exc, WindowsError):
+            print("Administrator permission required!",
+                  file=sys.stderr)
         traceback.print_exc(limit=3)
