@@ -4,8 +4,6 @@ _TIMEFORMAT = '%b %d %I:%M %p'
 import os
 import sys
 import tempfile
-
-from supervisor.helpers import DummyPopen
 from functools import total_ordering
 
 from supervisor.compat import Fault
@@ -70,6 +68,7 @@ class DummyOptions:
         tempdir = tempfile.gettempdir()
         self.identifier = 'supervisor'
         self.childlogdir = tempdir
+        self.delaysecs = 0.2
         self.uid = 999
         self.logger = self.getLogger()
         self.backofflimit = 10
@@ -273,7 +272,6 @@ class DummyOptions:
                 raise RuntimeError(self.execv_error)
         self.execv_args = (filename, argv)
         self.execv_environment = environment
-        return DummyPopen(filename, argv, environment)
 
     def drop_privileges(self, uid):
         if self.setuid_msg:
@@ -588,7 +586,9 @@ class DummyPConfig:
                  redirect_stderr=False,
                  stopsignal=None, stopwaitsecs=10, stopasgroup=False, killasgroup=False,
                  exitcodes=(0, ), environment=None, serverurl=None,
-                 systemjob=None):
+                 systemjob=None,
+                 socket_fd_param=None):
+        self.socket_fd_param = socket_fd_param
         self.options = options
         self.name = name
         self.command = command
